# coding=utf-8

import requests
import uuid
import hashlib
import rfc6266

from . import settings


class AtcfsApi(object):
    """
    Класс для работы с запросами к ATCFS
    """

    def _build_url(self, *args):
        """
        Функция составления полного урла.
        :param args: составные части пути
        :return: фбсолютный урл
        """
        chunks = (settings.URL,) + args
        url = u'/'.join(chunks)
        return url

    def _get_credential_headers(self):
        """
        Метод генерации данных для аутентификации на сервере ATCFS.
        :return: словарь с необходимыми для атунетификации полями
        """
        request_id = str(uuid.uuid4())
        sign = '{vis_id}_{vis_user}_{request_id}_{secret_key}'.format(**{
            'vis_id': settings.VIS_ID,
            'vis_user': settings.VIS_USER,
            'request_id': request_id,
            'secret_key': settings.SECRET_KEY,
        })
        sign = hashlib.md5(sign).hexdigest()
        headers = {
            'AtcFs-VisId': settings.VIS_ID,
            'AtcFs-VisUser': settings.VIS_USER,
            'AtcFs-RequestId': request_id,
            'AtcFs-Sign': sign,
        }
        return headers

    def upload_file(self, name, content):
        """
        Загрузка файла на сервер ATCFS
        :param name: название файла
        :param content: содержимое файла
        :return: идентификатор файла на сервере ATCFS
        """
        url = self._build_url(settings.FILES_PATH)
        params = {'fileName': name}
        data = content
        headers = self._get_credential_headers()
        headers['Content-Type'] = 'application/octet-stream'
        response = requests.post(
            url=url,
            params=params,
            data=data,
            headers=headers
        )
        if response.status_code != 201:
            raise Exception(response.text)
        ident = response.text
        return ident

    def download_file(self, ident):
        """
        Загружаем файл с сервера в память.
        :param ident: идентификатор файла
        :return: тюпл название и содержимое
        """
        url = self._build_url(settings.FILES_PATH, ident)
        headers = self._get_credential_headers()
        response = requests.get(
            url=url,
            headers=headers
        )
        if response.status_code != 200:
            raise Exception(response.text)
        content_disposition = rfc6266.parse_requests_response(response)
        file_name = content_disposition.filename_unsafe
        file_content = response.content
        return file_name, file_content

    def delete_file(self, ident):
        """
        Удаление файла на сервере ATCFS
        :param ident: идентификатор файла
        """
        url = self._build_url(settings.FILES_PATH, ident)
        headers = self._get_credential_headers()
        response = requests.delete(
            url=url,
            headers=headers
        )
        if response.status_code != 200:
            raise Exception(response.text)

    def get_file_url(self, ident):
        """
        Получить прямую ссылку на файл.
        :param ident: идентификатор файла
        :return: url
        """
        url = self._build_url(settings.TMP_FILE_LINK_PATH, ident)
        headers = self._get_credential_headers()
        response = requests.get(
            url=url,
            headers=headers
        )
        if response.status_code != 200:
            raise Exception(response.text)
        tmp_ident = response.text
        file_url = self._build_url(settings.TMP_FILES_PATH, tmp_ident)
        return file_url

    def get_file_info(self, ident):
        """
        Получить информацию о файле.
        :param ident: идентификатор файла
        :return: словарь с названием файла и его размером
        """
        url = self._build_url(settings.FILE_INFO_PATH, ident)
        headers = self._get_credential_headers()
        response = requests.get(
            url=url,
            headers=headers
        )
        if response.status_code != 200:
            raise Exception(response.text)
        file_json = response.json()
        file_info = {
            'name': file_json['fileName'],
            'size': file_json['size'],
        }
        return file_info
